package com.gihon.component.config;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;

import com.gihon.component.util.CommonContrants;

/**
 * Springboot 开启schedule 使用的线程池 默认是个单线程的
 * 
 * @author baihe
 * @date 2023/03/20
 */
@ConditionalOnProperty(prefix = "jiheng.common", havingValue = "true", matchIfMissing = false, name = "schedule")
@EnableScheduling
@Configuration
public class ScheduleConfig implements SchedulingConfigurer {
    /**
     * 容易造成线程爆满,最好定制实现一个ScheduledExecutorService
     */
    @Override
    public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
        ScheduledExecutorService scheduler =
            Executors.newScheduledThreadPool(CommonContrants.CPU_NUM * 2 + 1, new NameableThreadFactory("scheduled"));
        taskRegistrar.setScheduler(scheduler);
    }

    static class NameableThreadFactory implements ThreadFactory {
        private static final AtomicInteger poolNumber = new AtomicInteger(1);
        private final ThreadGroup group;
        private final AtomicInteger threadNumber = new AtomicInteger(1);
        private final String namePrefix;

        NameableThreadFactory(String name) {
            SecurityManager s = System.getSecurityManager();
            group = (s != null) ? s.getThreadGroup() : Thread.currentThread().getThreadGroup();
            namePrefix = name + "-pool-" + poolNumber.getAndIncrement() + "-thread-";
        }

        @Override
        public Thread newThread(Runnable r) {
            Thread t = new Thread(group, r, namePrefix + threadNumber.getAndIncrement(), 0);
            if (t.isDaemon())
                t.setDaemon(false);
            if (t.getPriority() != Thread.NORM_PRIORITY)
                t.setPriority(Thread.NORM_PRIORITY);
            return t;
        }
    }
}
