package com.gihon.component.web.auth;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.springframework.core.annotation.Order;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import com.gihon.component.enums.ModuleType;
import com.gihon.component.exception.LoginException;
import com.gihon.component.properties.GihonCommonProperties;
import com.gihon.component.properties.RedisConstants;
import com.gihon.component.response.ResponseStatus;
import com.gihon.component.web.service.TokenService;

import lombok.extern.slf4j.Slf4j;

/**
 * @description 认证拦截器 非Security
 * @author baihe
 *
 */
@Order(2)
@Slf4j
@Component
public class AuthenticationInterceptor implements HandlerInterceptor {

    @Resource
    private TokenService tokenService;
    @Resource
    private AuthService authService;
    @Resource
    private GihonCommonProperties gihonCommonProperties;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }

        String token = (String)request.getAttribute(AuthUtils.REQ_TOKEN);
        if (StringUtils.isBlank(token)) {
            // 未登录
            throw new LoginException(ResponseStatus.UNAUTHORIATION);
        }
        AuthUser user = AuthUtils.getUser();
        if (user == null) {
            // token 过期TOKEN_EXPIRED
            throw new LoginException(ResponseStatus.TOKEN_EXPIRED);

        }
        // 认证token是否合法
        // 超级管理员只要是web平台不校验权限
        if (AuthConstans.SUPER_ADMIN.equals(user.getUsername())
            && gihonCommonProperties.getModuleType() == ModuleType.WEB.getCode()) {
            return true;
        } else if (AuthConstans.SUPER_ADMIN.equals(user.getUsername())) {
            return false;
        }
        String url = request.getMethod() + RedisConstants.SEP + getUri(request);
        log.debug("url={},moudle={},token={},user={}", url, gihonCommonProperties.getModuleType(), token, user);
        // 动态鉴权
        boolean checkPermission = authService.checkPermission(url, gihonCommonProperties.getModuleType(), token, user);
        if (!checkPermission) {
            throw new LoginException(ResponseStatus.FORBIDDEN);
        }
        return true;
    }

    // 获取资源的路径,去掉ContextPath
    private String getUri(HttpServletRequest request) {
        String uri = request.getContextPath();
        String url = request.getRequestURI();
        // String url3 = request.getServletPath();
        if (uri.equals("/")) {
            return url;
        } else {
            return url.replace(uri, "");
        }
    }

}
