package com.gihon.component.web.auth.impl;

import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Service;
import org.springframework.util.AntPathMatcher;

import com.fasterxml.jackson.core.type.TypeReference;
import com.gihon.component.properties.GihonCommonProperties;
import com.gihon.component.properties.RedisConstants;
import com.gihon.component.util.JacksonJsonUtils;
import com.gihon.component.web.auth.AuthService;
import com.gihon.component.web.auth.AuthUser;
import com.gihon.component.web.service.TokenService;
import com.gihon.component.web.vo.PermissionVO;
import com.gihon.component.web.vo.TokenVal;
import com.github.xiaoymin.knife4j.core.util.CollectionUtils;

import lombok.extern.slf4j.Slf4j;

/**
 * 动态鉴权
 * 非Security
 * @author baihe
 *
 */
@Service
public class AuthServiceImpl implements AuthService{

    @Autowired
    private TokenService tokenService;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private GihonCommonProperties gihonCommonProperties;
    
   
    private AntPathMatcher antMatcher = new AntPathMatcher();
    
    /**
     * 通过请求地址和当前用户权限 获取请求地址需要的权限比较 忽略平台类型
     * 
     * @param uri
     * @param token
     * @param checkResult
     * @return
     */
    public boolean checkPermission(String uri, int moduleType, String token, AuthUser checkResult) {
        TokenVal tv = tokenService.getTokenVal(token);
        if (tv == null || tv.getModuleType() != moduleType) {
            return false;
        }
        // 获取全部的角色
        List<String> role = checkResult.getRoleList();

        // 获取全部的权限列表匹配
        // 用权限匹配角色,如果没有角色那么就通过
        List<String> needRole = getPermission(uri);
        // Null 是没有配置权限只要登录即可
        if (needRole == null) {
            return true;
        } else if (role == null||role.isEmpty()) {
            return false;
        }
        // Empty是配置权限但是没有角色 只有管理员才行 @See PermissionServiceImpl#cloneCopyMenu
        return needRole.stream().anyMatch(role::contains);
    }

    /**
     * 从redis缓存中读取权限
     * 
     * @return
     */
    public List<String> getPermission(String uri) {
        String[] uris = uri.split(RedisConstants.SEP);

        Comparator<String> c = antMatcher.getPatternComparator("/**");

        List<PermissionVO> list = getPermissionList();
        if(CollectionUtils.isEmpty(list)) {
            return null;
        }
        // 排序
        HttpMethod m = HttpMethod.valueOf(uris[0]);
        list = list.stream().filter(p -> m.matches(p.getMethod())).sorted((p, p2) -> {
            return c.compare(p.getUrl(), p2.getUrl());
        }).collect(Collectors.toList());

        for (PermissionVO vo : list) {
//			RequestMatcher matcher = new AntPathRequestMatcher(vo.getUrl(),vo.getMethod());
            if (antMatcher.match(vo.getUrl(), uris[1])) {
                return vo.getRoleList();
            }
        }
        return null;
    }

    /**
     * 从redis缓存中读取权限
     * @See AuthService.init()
     * @return
     */
    public List<PermissionVO> getPermissionList() {
        String hashKey = gihonCommonProperties.getModuleCode() + RedisConstants.SEP + gihonCommonProperties.getModuleType();
        String result = (String) stringRedisTemplate.opsForHash().get(RedisConstants.PERMISSN_REDIS, hashKey);
        if (StringUtils.isBlank(result)) {
            return null;
        }
        TypeReference<List<PermissionVO>> r = new TypeReference<List<PermissionVO>>() {
        };
        return JacksonJsonUtils.readObject(result, r);
    }

}
