package com.gihon.component.web.redis;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.listener.KeyExpirationEventMessageListener;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;

import com.gihon.component.web.service.TokenService;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class RedisKeyExpirationListener extends KeyExpirationEventMessageListener {


    @Autowired
    private TokenService tokenService;

    public RedisKeyExpirationListener(RedisMessageListenerContainer listenerContainer) {
        super(listenerContainer);
    }

    /**
     * redis key失效，监听
     * 
     * @param message
     * @param pattern
     */
    @Override
    public void onMessage(Message message, byte[] pattern) {
        
        String expiredKey = message.toString();
        log.debug("[失效RedisKey] [key:{}] [pattern:{}]", expiredKey, new String(pattern));
        
        if (expiredKey.startsWith(TokenService.REFRESH_TOKEN_PRE)) {
            
            String rkey = expiredKey.replace(TokenService.REFRESH_TOKEN_PRE, "");
            String userAccount = tokenService.getUserName(rkey);
            tokenService.autoClearRefreshToken(rkey, userAccount);
        }

    }
}
