package com.gihon.component.rbac.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gihon.component.rbac.entity.GihonButton;
import com.gihon.component.rbac.entity.GihonMenu;
import com.gihon.component.rbac.entity.GihonModule;
import com.gihon.component.rbac.service.GihonButtonService;
import com.gihon.component.rbac.service.GihonMenuService;
import com.gihon.component.rbac.service.GihonModuleService;
import com.gihon.component.rbac.vo.ModuleListReq;
import com.gihon.component.response.PageBean;
import com.gihon.component.response.Response;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(value ="系统菜单权限模块" ,tags = "系统菜单权限")
@RestController
@RequestMapping("/rbac/module")
public class ModuleController {

	@Autowired
	private GihonModuleService moduleService;
	@Autowired
	private GihonMenuService menuService;
	@Autowired
	private GihonButtonService buttonService;
	
	@ApiOperation("模块:列表")
	@GetMapping("list")
	public Response<PageBean<GihonModule>> list(ModuleListReq  moduleListReq) {
		IPage<GihonModule> list = moduleService.getModulelistPage(moduleListReq);
		return Response.okPage(list);
	}
	
	@ApiOperation("模块:新增")
	@PostMapping("")
	public Response<Void> saveModule(@RequestBody GihonModule module) {
		moduleService.saveModule(module);
		return Response.ok();
	}
	@ApiOperation("模块:详情")
	@GetMapping("/{id}")
	public Response<GihonModule> getModule(@PathVariable Long id) {
		return Response.ok(moduleService.getById(id));
	}
	@ApiOperation("模块:编辑")
	@PostMapping("/{id}")
	public Response<Void> updateModule(@RequestBody GihonModule module,@PathVariable Long id) {
		module.setId(id);
		moduleService.updateModule(module);
		return Response.ok();
	}
	@ApiOperation("模块:删除")
	@DeleteMapping("/{id}")
	public Response<Boolean> removeModule(@PathVariable Long id) {
		return Response.ok(moduleService.removeById(id));
	}
	@ApiOperation("模块:全部列表")
	@GetMapping("all")
	public Response<List<GihonModule>> list() {
		List<GihonModule> list = moduleService.list();
		return Response.ok(list);
	}
	/**************菜单**********************/
	@ApiOperation("菜单:列表")
	@GetMapping("menu/list")
	public Response<List<GihonMenu>> listMenu(@RequestParam Long moduleId,@RequestParam(required = false) Long menuId) {
		if(menuId==null) {
			menuId = -1L;
		}
		List<GihonMenu> list = menuService.getSubMenu(moduleId,menuId,false);
		return Response.ok(list);
	}
	
	@ApiOperation("菜单:新增")
	@PostMapping("menu")
	public Response<Void> saveMenu(@RequestBody GihonMenu menu) {
		menuService.saveMenu(menu);
		return Response.ok();
	}
	@ApiOperation("菜单:编辑")
	@PostMapping("menu/{id}")
	public Response<Void> updateMenu(@RequestBody GihonMenu menu,@PathVariable Long id) {
		menu.setId(id);
		menuService.updateMenu(menu);
		return Response.ok();
	}
	@ApiOperation("菜单:详情")
	@GetMapping("menu/{id}")
	public Response<GihonMenu> getMenu(@PathVariable Long id) {
		return Response.ok(menuService.getById(id));
	}
	@ApiOperation("菜单:删除")
	@DeleteMapping("menu/{id}")
	public Response<Boolean> removeMenu(@PathVariable Long id) {
		return Response.ok(menuService.removeById(id));
	}
	/***************权限按钮******************/
	@ApiOperation("按钮权限:列表")
	@GetMapping("button/list")
	public Response<List<GihonButton>> listButton(@RequestParam Long menuId) {
		List<GihonButton> list = buttonService.getMenuButton(menuId);
		return Response.ok(list);
	}
	
	@ApiOperation("按钮权限:新增")
	@PostMapping("button")
	public Response<Void> saveButton(@RequestBody GihonButton bottun) {
		buttonService.saveButton(bottun);
		return Response.ok();
	}
	@ApiOperation("按钮权限:编辑")
	@PostMapping("button/{id}")
	public Response<Void> updateButton(@RequestBody GihonButton bottun,@PathVariable Long id) {
		bottun.setId(id);
		buttonService.updateButton(bottun);
		return Response.ok();
	}
	@ApiOperation("按钮权限:详情")
	@GetMapping("button/{id}")
	public Response<GihonButton> getButton(@PathVariable Long id) {
		return Response.ok(buttonService.getById(id));
	}
	@ApiOperation("按钮权限:删除")
	@DeleteMapping("button/{id}")
	public Response<Boolean> removeButton(@PathVariable Long id) {
		return Response.ok(buttonService.removeById(id));
	}
	
	
}
