package com.gihon.component.rbac.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gihon.component.rbac.entity.GihonButton;
import com.gihon.component.rbac.entity.GihonRole;
import com.gihon.component.rbac.service.GihonRoleService;
import com.gihon.component.rbac.vo.RoleButtonReq;
import com.gihon.component.rbac.vo.RoleListReq;
import com.gihon.component.response.PageBean;
import com.gihon.component.response.Response;
import com.gihon.component.response.SelectVal;
import com.gihon.component.web.auth.AuthUtils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(value = "角色模块", tags = "角色模块")
@RestController
@RequestMapping("/rbac/role")
public class RoleController {

	@Autowired
	private GihonRoleService roleService;

	@ApiOperation("ROLE:角色列表")
	@GetMapping("list")
	public Response<PageBean<GihonRole>> list(RoleListReq roleListReq) {
	    roleListReq.setCompanyId(AuthUtils.getCompanyId());
		IPage<GihonRole> list = roleService.getRolelistPage(roleListReq);
		return Response.okPage(list);
	}

	@ApiOperation("ROLE:新增")
	@PostMapping("")
	public Response<Void> saveRole(@RequestBody GihonRole role) {
		roleService.saveRole(role);
		return Response.ok();
	}
	
	@ApiOperation("ROLE:详情")
	@GetMapping("/{id}")
	public Response<GihonRole> saveRole(@PathVariable Long id) {
		return Response.ok(roleService.getById(id));
	}

	@ApiOperation("ROLE:编辑")
	@PostMapping("/{id}")
	public Response<Void> updateRole(@RequestBody GihonRole role, @PathVariable Long id) {
		role.setId(id);
		roleService.updateRole(role);
		return Response.ok();
	}

	@ApiOperation("ROLE:删除")
	@DeleteMapping("/{id}")
	public Response<Void> removeRole(@PathVariable Long id) {
		roleService.removeRole(id);
		return Response.ok();
	}

    @ApiOperation("ROLE:列表")
    @GetMapping("/labelAndValue")
    public Response<List<SelectVal>> labelAndValue(Long comanyId,String roleName) {
        return Response.ok(roleService.labelAndValue(comanyId,roleName));
    }
    
	@ApiOperation("ROLE:权限列表")
	@PostMapping("button/list")
	public Response<List<GihonButton>> getButtonByRoleModule(@RequestParam Long roleId, @RequestParam Long moduleId) {
		return Response.ok(roleService.getButtonByRoleModule(roleId, moduleId));
	}

	@ApiOperation("ROLE:设置权限列表")
	@PostMapping("button/set")
	public Response<Void> setButtonByRoleModule(@RequestBody RoleButtonReq req) {
		roleService.setButtonByRoleModule(req);
		return Response.ok();
	}
	
}
