package com.gihon.component.rbac.controller;

import java.util.List;

import javax.validation.constraints.NotNull;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.rbac.entity.GihonUser;
import com.gihon.component.rbac.service.GihonUserService;
import com.gihon.component.rbac.vo.UserListReq;
import com.gihon.component.rbac.vo.UserRoleReq;
import com.gihon.component.response.PageBean;
import com.gihon.component.response.Response;
import com.gihon.component.response.ResponseStatus;
import com.gihon.component.web.auth.AuthType;
import com.gihon.component.web.auth.AuthUtils;
import com.gihon.component.web.auth.PreAuth;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@Api(value = "人员模块", tags = "人员模块")
@Slf4j
@RestController
@RequestMapping("/rbac/user")
public class UserController {
    @Autowired
    private GihonUserService gihonUserService;

    @ApiOperation("USER:用户列表")
    @GetMapping("list")
    public Response<PageBean<GihonUser>> list(@Validated UserListReq search) {
        if (search.getCompanyId() == null) {
            search.setCompanyId(AuthUtils.getCompanyId());
        }
        IPage<GihonUser> list = gihonUserService.getUserlistPage(search);
        return Response.okPage(list);
    }

    @ApiOperation("USER:启用管理员")
    @PostMapping("enable/{id}")
    public Response<Void> enable(@PathVariable Long id) {
        gihonUserService.enableCompany(id, true);
        return Response.ok();
    }

    @ApiOperation("USER:禁用管理员")
    @PostMapping("disable/{id}")
    public Response<Void> disable(@PathVariable Long id) {
        gihonUserService.enableCompany(id, false);
        return Response.ok();
    }

    @ApiOperation("USER:用户详情")
    @GetMapping("{id}")
    public Response<GihonUser> user(@Validated @PathVariable Long id) {
        return Response.ok(gihonUserService.getById(id));
    }

    @ApiOperation("USER:新增")
    @PostMapping("")
    public Response<Void> saveUser(@Validated @RequestBody GihonUser user) {
        Long companyId = AuthUtils.getCompanyId();
        if (companyId != null && !companyId.equals(user.getCompanyId())) {
            throw new BusinessException(ResponseStatus.FORBIDDEN);
        }
        gihonUserService.saveUser(user);
        return Response.ok();
    }

    @ApiOperation("USER:编辑")
    @PostMapping("/{id}")
    public Response<Void> updateUser(@Validated @RequestBody GihonUser user, @PathVariable Long id) {
        Long companyId = AuthUtils.getCompanyId();
        if (companyId != null && !companyId.equals(user.getCompanyId())) {
            throw new BusinessException(ResponseStatus.FORBIDDEN);
        }
        user.setId(id);
        gihonUserService.updateUser(user);
        return Response.ok();
    }
    
    @PreAuth(authType = AuthType.AUTH,btnCode = "user_edit")
    @ApiOperation("USER:用户获取角色")
    @GetMapping("role/get")
    public Response<List<Long>> getUserRole(@RequestParam Long userId) {
        return Response.ok(gihonUserService.getUserRole(userId));
    }
    
    @PreAuth(authType = AuthType.AUTH,btnCode = "user_edit")
    @ApiOperation("USER:用户设置角色")
    @PostMapping("role/set")
    public Response<Void> setUserRole(@Validated @RequestBody UserRoleReq userRole) {
        gihonUserService.setUserRole(userRole);
        return Response.ok();
    }
    @PreAuth(authType = AuthType.AUTH,btnCode = "user_reset")
    @ApiOperation("USER:用户重置密码")
    @PostMapping("reset")
    public Response<Void> resetPassword(@NotNull @RequestParam Long userId) {
        gihonUserService.resetPassword(userId);
        return Response.ok();
    }
}
