package com.gihon.component.rbac.entity;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;

import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.gihon.component.entity.CompanyBaseEntity;
import com.gihon.component.web.vo.UserInfo;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

/**
 * 系统账号
 * 
 * @author baihe
 *
 */
@Setter
@Getter
@TableName("gihon_user")
public class GihonUser extends CompanyBaseEntity {

    @NotBlank(message="昵称不能为空")
    @Size(max = 16 ,message="昵称不超过16位")
    @ApiModelProperty(value = "名称")
    private String nickName;

    @NotBlank(message="账号不能为空")
    @Size(max = 16 ,message="账号不超过16位")
    @ApiModelProperty(value = "账号")
    private String username;

    @Size(max = 16 ,message="电话不超过16位")
    @ApiModelProperty(value = "电话")
    private String telephone;

    @JsonIgnore
    @ApiModelProperty(value = "密码")
    private String password;

    /**
     * @see com.gihon.component.enums.Gender
     */
    @Max(value = 2, message = "性别取值范围1-2")
    @Min(value = 1, message = "性别取值范围1-2")
    @ApiModelProperty(value = "性别1男2女")
    private Integer gender;

    /**
     * @see com.gihon.component.enums.Gender
     */
    @ApiModelProperty(value = "是否是公司管理员")
    private Boolean companyAdmin = false;
    
    public UserInfo toUserInfo() {
        UserInfo userInfo = new UserInfo();
        userInfo.setId(this.id);
        userInfo.setUsername(this.username);
        userInfo.setNickName(this.nickName);
        userInfo.setPassword(this.password);
        userInfo.setSalt(this.username);
        userInfo.setCompanyId(this.companyId);
        userInfo.setCompanyAdmin(this.companyAdmin);
        return userInfo;
    }
}
