package com.gihon.component.rbac.service;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.rbac.entity.GihonUser;
import com.gihon.component.rbac.vo.UserListReq;
import com.gihon.component.rbac.vo.UserRoleReq;
import com.gihon.component.util.PasswordUtils;

public interface GihonUserService extends IService<GihonUser> {

    IPage<GihonUser> getUserlistPage(UserListReq search);

    default boolean enableCompany(Long userId, boolean enableFlag) {
        return this.lambdaUpdate().eq(GihonUser::getId, userId).eq(GihonUser::getCompanyAdmin, !enableFlag)
            .set(GihonUser::getCompanyAdmin, enableFlag).update();
    }

    /**
     * 新增
     */
    default boolean saveUser(GihonUser user) {
        int count = this.lambdaQuery().eq(GihonUser::getUsername, user.getUsername()).count();
        if (count > 0) {
            throw new BusinessException("账号重复");
        }
        user.setState(0);
        user.setCompanyAdmin(false);
        user.setPassword(PasswordUtils.makePassword("123456", user.getUsername()));
        return this.save(user);

    }

    /**
     * 更新
     */
    default boolean updateUser(GihonUser user) {
        GihonUser compnayOld = this.getById(user.getId());
        if (compnayOld == null) {
            throw new BusinessException("账号不存在");
        }

        GihonUser userUpdate = new GihonUser();
        userUpdate.setId(user.getId());
        userUpdate.setNickName(user.getNickName());
        userUpdate.setTelephone(user.getTelephone());
        userUpdate.setGender(user.getGender());
        return this.updateById(userUpdate);

    }

    /**
     * 重置密码
     */
    default boolean resetPassword(Long userId) {
        GihonUser user = this.getById(userId);
        if (user == null) {
            throw new BusinessException("账号不存在");
        }
        GihonUser userUpdate = new GihonUser();
        userUpdate.setId(user.getId());
        userUpdate.setPassword(PasswordUtils.makePassword("123456", user.getUsername()));

        return this.updateById(userUpdate);

    }

    boolean setUserRole(UserRoleReq userRole);

    List<Long> getUserRole(Long userId);

}
