package com.gihon.component.rbac.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.rbac.entity.GihonButton;
import com.gihon.component.rbac.entity.GihonMenu;
import com.gihon.component.rbac.listener.CustomePushlisher;
import com.gihon.component.rbac.listener.CustomerEvent;
import com.gihon.component.rbac.mapper.GihonButtonMapper;
import com.gihon.component.rbac.mapper.GihonMenuMapper;
import com.gihon.component.rbac.service.GihonButtonService;
import com.google.common.collect.Lists;

/**
 * 直接继承没有多实现处理,这样不能用事务注解
 * 
 * @author baihe
 *
 */
@Service
public class GihonButtonServiceImpl extends ServiceImpl<GihonButtonMapper, GihonButton> implements GihonButtonService {

    @Autowired
    protected CustomePushlisher customePushlisher;

    @Autowired
    private GihonMenuMapper gihonMenuMapper;

    /**
     * 获取菜单下按钮权限
     */
    public List<GihonButton> getMenuButton(Long menuId) {
        return this.lambdaQuery().eq(GihonButton::getMenuId, menuId).orderByAsc(GihonButton::getOrderNum).list();
    }

    /**
     * 获取菜单列表下按钮权限
     */
    public List<GihonButton> getMenuButton(List<Long> menuIdList, boolean idOnly) {
        if (menuIdList.isEmpty()) {
            return Lists.newArrayList();
        }
        LambdaQueryChainWrapper<GihonButton> wrapper =
            this.lambdaQuery().in(GihonButton::getMenuId, menuIdList).orderByAsc(GihonButton::getOrderNum);
        if (idOnly) {
            wrapper.select(GihonButton::getId);
        }
        return wrapper.list();
    }

    /**
     * 新增按钮权限
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean saveButton(GihonButton button) {
        int count = this.lambdaQuery().eq(GihonButton::getMenuId, button.getMenuId()).and(p -> {
            p.eq(GihonButton::getButtonName, button.getButtonName()).or().eq(GihonButton::getButtonCode,
                button.getButtonCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("按钮编码或者名称重复");
        }
        GihonMenu menu = gihonMenuMapper.selectById(button.getMenuId());
        CustomerEvent event = new CustomerEvent(menu.getModuleId());
        event.setType(3);
        event.setId(menu.getModuleId());
        customePushlisher.publish(event);
        
        return this.save(button);

    }

    /**
     * 更新按钮权限
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateButton(GihonButton button) {
        GihonButton buttonOld = this.getById(button.getId());
        if (buttonOld == null) {
            throw new BusinessException("按钮不存在");
        }
        int count = this.lambdaQuery().eq(GihonButton::getMenuId, buttonOld.getMenuId())
            .ne(GihonButton::getId, button.getId()).and(p -> {
                p.eq(GihonButton::getButtonName, button.getButtonName()).or().eq(GihonButton::getButtonCode,
                    button.getButtonCode());
            }).count();
        if (count > 0) {
            throw new BusinessException("按钮编码或者名称重复");
        }
        button.setMenuId(buttonOld.getMenuId());

        return this.updateById(button);

    }

    @Transactional(rollbackFor = Exception.class)
    public boolean removeEntity(Long id) {
        GihonButton buttonOld = this.getById(id);
        if (buttonOld == null) {
            throw new BusinessException("按钮权限不存在");
        }
        this.removeById(id);
        GihonMenu menu = gihonMenuMapper.selectById(buttonOld.getMenuId());
        CustomerEvent event = new CustomerEvent(menu.getModuleId());
        event.setType(3);
        event.setId(menu.getModuleId());
        customePushlisher.publish(event);
        return true;
    }

}
