package com.gihon.component.rbac.service.impl;

import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.rbac.entity.GihonModule;
import com.gihon.component.rbac.mapper.GihonModuleMapper;
import com.gihon.component.rbac.service.GihonModuleService;
import com.gihon.component.rbac.vo.ModuleListReq;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonModuleServiceImpl extends ServiceImpl<GihonModuleMapper, GihonModule> implements GihonModuleService{

    /**
     * 按照code获取模块
     */
    public GihonModule getModuleByCode(String code) {
        return this.lambdaQuery().eq(GihonModule::getModuleCode, code).one();
    }

    /**
     * 分页获取模块
     */
    public IPage<GihonModule> getModulelistPage(ModuleListReq moduleListReq) {
        Page<GihonModule> page = new Page<GihonModule>(moduleListReq.getPageIndex(), moduleListReq.getPageSize());
        return this.lambdaQuery().like(StringUtils.hasText(moduleListReq.getModuleName()), GihonModule::getModuleName, moduleListReq.getModuleName())
                .like(StringUtils.hasText(moduleListReq.getModuleCode()), GihonModule::getModuleCode, moduleListReq.getModuleCode()).page(page);
    }

    /**
     * 新增模块
     */
    public boolean saveModule(GihonModule module) {
        int count = this.lambdaQuery().eq(GihonModule::getModuleName, module.getModuleName()).or().eq(GihonModule::getModuleCode, module.getModuleCode())
                .count();
        if (count > 0) {
            throw new BusinessException("模块编码或者名称重复");
        }
        return this.save(module);

    }

    /**
     * 编辑模块
     */
    public boolean updateModule(GihonModule module) {
        GihonModule moduleOld = this.getById(module.getId());
        if (moduleOld == null) {
            throw new BusinessException("模块不存在");
        }
        int count = this.lambdaQuery().ne(GihonModule::getId, module.getId()).and(p -> {
            p.eq(GihonModule::getModuleName, module.getModuleName()).or().eq(GihonModule::getModuleCode, module.getModuleCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("模块编码或者名称重复");
        }
        GihonModule moduleUpdate = new GihonModule();
        moduleUpdate.setId(module.getId());
        moduleUpdate.setModuleCode(module.getModuleCode());
        moduleUpdate.setModuleName(module.getModuleName());
        moduleUpdate.setOrderNum(module.getOrderNum());
        moduleUpdate.setModuleDescription(module.getModuleDescription());
        moduleUpdate.setModuleLogo(module.getModuleLogo());

        return this.updateById(moduleUpdate);

    }
}
