package com.gihon.component.rbac.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.rbac.entity.GihonRoleButton;
import com.gihon.component.rbac.listener.CustomePushlisher;
import com.gihon.component.rbac.mapper.GihonRoleButtonMapper;
import com.gihon.component.rbac.service.GihonRoleButtonService;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonRoleButtonServiceImpl extends ServiceImpl<GihonRoleButtonMapper, GihonRoleButton> implements GihonRoleButtonService{

    /**
     * 获取按钮关联的角色
     */
    public List<Long> getRoleListByBtn(Long buttonId) {
        List<GihonRoleButton> list = this.lambdaQuery().eq(GihonRoleButton::getButtonId, buttonId).select(GihonRoleButton::getRoleId).list();
        return list.stream().map(rb -> rb.getRoleId()).collect(Collectors.toList());
    }

    /**
     * 获取角色关联的按钮
     */
    public List<Long> getButtonListByRole(Long roleId) {
        List<GihonRoleButton> list = this.lambdaQuery().eq(GihonRoleButton::getRoleId, roleId).select(GihonRoleButton::getButtonId).list();
        return list.stream().map(rb -> rb.getButtonId()).collect(Collectors.toList());
    }

    /**
     * 移除角色关联的按钮(buttonList为null的时候是删除全部,否则只是部分删除)
     */
    public boolean removeButtonByRoleId(Long roleId, List<Long> buttonList) {
        if (buttonList != null && buttonList.isEmpty()) {
            return false;
        }
        return this.lambdaUpdate().eq(GihonRoleButton::getRoleId, roleId).in(buttonList != null, GihonRoleButton::getButtonId, buttonList).remove();
    }

    /**
     * 新增角色关联的按钮
     */
    public boolean insertRoleButton(Long roleId, List<Long> buttonList) {
        if (buttonList == null || buttonList.isEmpty()) {
            return false;
        }
        return this.baseMapper.insertRoleButton(roleId, buttonList);
    }

    /**
     * 移除角色关联的按钮
     */
    public boolean deleteRoleButton(Long roleId, List<Long> buttonList) {
        if (buttonList == null || buttonList.isEmpty()) {
            return false;
        }
        return this.baseMapper.deleteRoleButton(roleId, buttonList);
    }
}
