package com.gihon.component.rbac.service.impl;

import java.util.List;
import java.util.stream.Collectors;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.rbac.entity.GihonUserRole;
import com.gihon.component.rbac.mapper.GihonUserRoleMapper;
import com.gihon.component.rbac.service.GihonUserRoleService;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonUserRoleServiceImpl extends ServiceImpl<GihonUserRoleMapper, GihonUserRole>
    implements GihonUserRoleService {

    /**
     * 获取某个用户的全部角色
     */
    public List<Long> roleListByUserId(Long id) {
        List<GihonUserRole> roleList =
            this.lambdaQuery().eq(GihonUserRole::getUserId, id).select(GihonUserRole::getRoleId).list();
        return roleList.stream().map(ur -> ur.getRoleId()).collect(Collectors.toList());
    }

    /**
     * 删除某个用户的全部角色
     */
    public boolean removeRoleByUserId(Long id, List<Long> roleList) {
        if (roleList != null && roleList.isEmpty()) {
            return false;
        }
        return this.lambdaUpdate().eq(GihonUserRole::getUserId, id)
            .in(roleList != null, GihonUserRole::getRoleId, roleList).remove();
    }

    /**
     * 新增角色关联的按钮
     */
    public boolean insertUserRole(Long id, List<Long> roleList) {
        if (roleList == null || roleList.isEmpty()) {
            return false;
        }
        return this.baseMapper.insertUserRole(id, roleList);
    }

}
