package com.gihon.component.rbac.service.impl;

import java.util.Iterator;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.rbac.entity.GihonCompany;
import com.gihon.component.rbac.entity.GihonUser;
import com.gihon.component.rbac.listener.CustomePushlisher;
import com.gihon.component.rbac.listener.CustomerEvent;
import com.gihon.component.rbac.mapper.GihonUserMapper;
import com.gihon.component.rbac.service.GihonCompanyService;
import com.gihon.component.rbac.service.GihonUserRoleService;
import com.gihon.component.rbac.service.GihonUserService;
import com.gihon.component.rbac.vo.UserListReq;
import com.gihon.component.rbac.vo.UserRoleReq;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonUserServiceImpl extends ServiceImpl<GihonUserMapper, GihonUser> implements GihonUserService {

    @Autowired
    protected CustomePushlisher customePushlisher;
    
    @Autowired
    private GihonCompanyService gihonCompanyService;
    @Autowired
    private GihonUserRoleService gihonUserRoleService;
    
    public IPage<GihonUser> getUserlistPage(UserListReq search) {
        Page<GihonUser> page = new Page<>(search.getPageIndex(), search.getPageSize());
        IPage<GihonUser> result = this.lambdaQuery()
            .like(StringUtils.hasText(search.getUsername()), GihonUser::getUsername, search.getUsername())
            .eq(search.getCompanyId() != null, GihonUser::getCompanyId, search.getCompanyId())
            .eq(search.getCompanyAdmin() != null, GihonUser::getCompanyAdmin, search.getCompanyAdmin()).page(page);
        result.getRecords().forEach(u->{
            if(u.getCompanyId()!=null) {
                GihonCompany c = gihonCompanyService.getById(u.getCompanyId());
                if (c != null) {
                    u.setCompanyName(c.getCompanyName());
                }
            }
        }); 
        return result;
    }
    @Override
    public boolean setUserRole(UserRoleReq userRole) {

        List<Long> roleList = gihonUserRoleService.roleListByUserId(userRole.getUserId());
        Iterator<Long> it = roleList.iterator();
        while (it.hasNext()) {
            Long id = it.next();
            if (userRole.getRoleList().remove(id)) {
                it.remove();
            }
        }

        if (!userRole.getRoleList().isEmpty()) {
            gihonUserRoleService.insertUserRole(userRole.getUserId(), userRole.getRoleList());
        }
        if (!roleList.isEmpty()) {
            gihonUserRoleService.removeRoleByUserId(userRole.getUserId(), roleList);
        }
        // TODO 通知权限刷新
        CustomerEvent event = new CustomerEvent(userRole.getUserId());
        event.setType(1);
        event.setId(userRole.getUserId());
        customePushlisher.publish(event);
        return true;
    }

    @Override
    public List<Long> getUserRole(Long userId) {
        return gihonUserRoleService.roleListByUserId(userId);
    }
}
